package com.stereodustparticles.mooler_caster_console;

// SDP Mooler Caster Console
// Ridiculously simple radio DJ program, with a 16-slot soundboard and 2-deck music player
// Written by Ben A. (IfYouLikeGoodIdeas)

// Uses MP3SPI, VorbisSPI, and jFLAC to provide MP3, Vorbis, and FLAC support

// NOTE: jFLAC 1.3's jar file won't work out of the box, as it doesn't contain the proper SPI metadata.
// Use the provided "spi-fix" jar instead.

// NOTE: You must use tritonus_share version 0.3.7-2 (or higher, perhaps) for Vorbis decoding to work!

import java.awt.EventQueue;

import javax.swing.JFrame;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.UnsupportedAudioFileException;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

import java.awt.Font;
import java.awt.Color;
import javax.swing.JTextField;

import javax.swing.JProgressBar;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.awt.event.ActionEvent;
import javax.swing.JSeparator;

public class SDPConsole {
	
	public static final String PROG_VERSION = "1.03";
	
	static boolean spotLoading = false;
	static File lastSpotPath = null;
	static File lastDeckPath = null;
	
	// GUI stuff (most generated by WindowBuilder)
	private static JFrame SDPMain;
	private static JTextField deck1Artist;
	private static JTextField deck1Title;
	private static JTextField deck2Title;
	private static JTextField deck2Artist;
	private static JTextField deck1Time;
	private static JTextField deck2Time;
	private static JButton spotLoad;
	private static Spot[] spots = new Spot[16];
	private static Deck[] decks = {new Deck(), new Deck()};

	/**
	 * "Main" method - Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					initialize();
					SDPConsole.SDPMain.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Initialize the contents of the frame.
	 * Most of this was generated by Eclipse WindowBuilder, so it probably won't be terribly readable...
	 * The good stuff is below all of this...
	 */
	private static void initialize() {
		SDPMain = new JFrame();
		SDPMain.setResizable(false);
		SDPMain.setTitle("SDP Mooler Caster Console");
		SDPMain.setBounds(100, 100, 411, 680);
		SDPMain.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		SDPMain.getContentPane().setLayout(null);
		
		// Generate spot buttons for soundboard
		// TODO Recall previous configuration
		// Note that these buttons don't show up in the designer
		for ( int i = 0; i < 16; i++ ) {
			spots[i] = new Spot(i);
			SDPMain.getLayeredPane().add(spots[i].getButton(), 0, 0);
			SDPMain.getLayeredPane().add(spots[i].getCounter(), 1, 0);
		}
		
		// Deck 1 Label
		JLabel lblDeck1 = new JLabel("Deck 1:"); // Instantiate and set text
		lblDeck1.setBounds(10, 448, 70, 14); // Set position and size
		SDPMain.getContentPane().add(lblDeck1); // Add it to the window
		
		// Deck 2 Label
		JLabel lblDeck2 = new JLabel("Deck 2:");
		lblDeck2.setBounds(10, 550, 70, 14);
		SDPMain.getContentPane().add(lblDeck2);
		
		// Deck 1 Play Button
		JButton deck1Play = new JButton("PLAY");
		deck1Play.addActionListener(new ActionListener() { // Add an event handler
			// Method to execute when the button is clicked
			public void actionPerformed(ActionEvent e) {
				decks[0].playButton();
			}
		});
		deck1Play.setBackground(Color.GREEN);
		deck1Play.setFont(new Font("Tahoma", Font.BOLD, 18));
		deck1Play.setBounds(10, 467, 89, 72);
		SDPMain.getContentPane().add(deck1Play);
		
		// Deck 2 Play Button
		JButton deck2Play = new JButton("PLAY");
		deck2Play.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				decks[1].playButton();
			}
		});
		deck2Play.setBackground(Color.GREEN);
		deck2Play.setFont(new Font("Tahoma", Font.BOLD, 18));
		deck2Play.setBounds(10, 568, 89, 72);
		SDPMain.getContentPane().add(deck2Play);
		
		// Deck 1 Load Button
		JButton deck1Load = new JButton("LOAD");
		deck1Load.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				loadDeck(0);
			}
		});
		deck1Load.setBackground(Color.ORANGE);
		deck1Load.setBounds(109, 516, 74, 23);
		SDPMain.getContentPane().add(deck1Load);
		
		// Deck 2 Load Button
		JButton deck2Load = new JButton("LOAD");
		deck2Load.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				loadDeck(1);
			}
		});
		deck2Load.setBackground(Color.ORANGE);
		deck2Load.setBounds(109, 617, 74, 23);
		SDPMain.getContentPane().add(deck2Load);
		
		// "Load Spot" Button
		spotLoad = new JButton("Load Spot");
		spotLoad.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				// Toggle spot loading mode
				if ( ! spotLoading ) {
					spotLoading = true;
					spotLoad.setText("Cancel");
					spotLoad.setBackground(Color.RED);
					setAllSpotColors(Color.ORANGE);
				}
				else {
					endSpotLoading();
				}
			}
		});
		spotLoad.setBackground(Color.ORANGE);
		spotLoad.setBounds(139, 410, 124, 23);
		SDPMain.getContentPane().add(spotLoad);
		
		// Deck 1 Artist Box
		deck1Artist = new JTextField();
		deck1Artist.setEditable(false);
		deck1Artist.setText("No track loaded.");
		deck1Artist.setBounds(109, 467, 229, 20);
		SDPMain.getContentPane().add(deck1Artist);
		deck1Artist.setColumns(10);
		
		// Deck 1 Title Box
		deck1Title = new JTextField();
		deck1Title.setText("No track loaded.");
		deck1Title.setEditable(false);
		deck1Title.setColumns(10);
		deck1Title.setBounds(109, 492, 286, 20);
		SDPMain.getContentPane().add(deck1Title);
		
		// Deck 2 Title Box
		deck2Title = new JTextField();
		deck2Title.setText("No track loaded.");
		deck2Title.setEditable(false);
		deck2Title.setColumns(10);
		deck2Title.setBounds(109, 593, 286, 20);
		SDPMain.getContentPane().add(deck2Title);
		
		// Deck 2 Artist Box
		deck2Artist = new JTextField();
		deck2Artist.setText("No track loaded.");
		deck2Artist.setEditable(false);
		deck2Artist.setColumns(10);
		deck2Artist.setBounds(109, 568, 229, 20);
		SDPMain.getContentPane().add(deck2Artist);
		
		// Deck 1 Time Box
		deck1Time = new JTextField();
		deck1Time.setEditable(false);
		deck1Time.setText("0:00.0");
		deck1Time.setBounds(345, 467, 50, 20);
		SDPMain.getContentPane().add(deck1Time);
		deck1Time.setColumns(10);
		
		// Deck 2 Time Box
		deck2Time = new JTextField();
		deck2Time.setText("0:00.0");
		deck2Time.setEditable(false);
		deck2Time.setColumns(10);
		deck2Time.setBounds(345, 568, 50, 20);
		SDPMain.getContentPane().add(deck2Time);
		
		// Deck 1 Progress Bar
		JProgressBar deck1Progress = new JProgressBar();
		deck1Progress.setBounds(193, 516, 202, 23);
		SDPMain.getContentPane().add(deck1Progress);
		
		// Deck 2 Progress Bar
		JProgressBar deck2Progress = new JProgressBar();
		deck2Progress.setBounds(193, 617, 202, 23);
		SDPMain.getContentPane().add(deck2Progress);
		
		// Separator Bar between Spots and Decks
		JSeparator separator = new JSeparator();
		separator.setBounds(10, 444, 385, 2);
		SDPMain.getContentPane().add(separator);
		
		// Associate deck controls/fields with their corresponding Deck objects
		decks[0].associateFields(deck1Play, deck1Title, deck1Artist, deck1Time, deck1Progress);
		decks[1].associateFields(deck2Play, deck2Title, deck2Artist, deck2Time, deck2Progress);
	}
	
	// Pop up a file chooser dialog, then load the selected file into the specified deck
	// (Return without loading if Cancel was clicked)
	// Takes the deck index (0 or 1) as its parameter
	public static void loadDeck(int deck) {
		JFileChooser chooser = new JFileChooser();
		
		// Check if a directory was stored from a previous load operation, and if so,
		// move the file chooser to that directory
		if ( SDPConsole.lastDeckPath != null ) {
			chooser.setCurrentDirectory(SDPConsole.lastDeckPath);
		}
		
		// Open the file chooser dialog, and store the response code from the operation
		int response = chooser.showOpenDialog(SDPMain);
		
		// If user clicked Cancel, return without changing anything
		if ( response == JFileChooser.CANCEL_OPTION ) {
			return;
		}
		
		// Otherwise, try to load the file
		try {
			// Load the file into the deck
			decks[deck].load(chooser.getSelectedFile());
			
			// Store the directory the file was loaded from
			lastDeckPath = chooser.getCurrentDirectory();
		}
		// Error messages!
		catch (UnsupportedAudioFileException e) {
			JOptionPane.showMessageDialog(SDPConsole.getMainWindow(), "The file you selected is of an unsupported format", "Error Loading Deck", JOptionPane.ERROR_MESSAGE);
		}
		catch (IllegalArgumentException e) {
			JOptionPane.showMessageDialog(SDPConsole.getMainWindow(), "The output line could not be opened.\n\nCheck that your audio output is working properly, and that the file is of a supported format.", "Error Loading Deck", JOptionPane.ERROR_MESSAGE);
			e.printStackTrace();
		}
		catch (IOException e) {
			JOptionPane.showMessageDialog(SDPConsole.getMainWindow(), "Could not load the deck: " + e.getMessage() + "\n\nThe file may be corrupt.  Microwave it, then try again.", "Error Loading Deck", JOptionPane.ERROR_MESSAGE);
			e.printStackTrace();
		}
		catch (LineUnavailableException e) {
			JOptionPane.showMessageDialog(SDPConsole.getMainWindow(), "The audio line is unavailable for some reason.  Try clubbing your computer.", "Error Loading Deck", JOptionPane.ERROR_MESSAGE);
		}
	}
	
	// Batch-set the background colors of all the spot buttons
	public static void setAllSpotColors(Color color) {
		for ( Spot spot : spots ) {
			spot.getButton().setBackground(color);
		}
	}
	
	// Return the main window (so methods from other classes can show dialogs)
	public static JFrame getMainWindow() {
		return SDPMain;
	}
	
	// Return the soundboard to its normal state (from the loading state)
	public static void endSpotLoading() {
		spotLoad.setBackground(Color.ORANGE);
		spotLoad.setText("Load Spot");
		setAllSpotColors(null);
		spotLoading = false;
	}
}
